@@ -6,9 +6,12 @@
 Helpers that make development with ``structlog`` more pleasant.
 """
 
-from __future__ import absolute_import, division, print_function
+import sys
 
-from six import PY2, StringIO, string_types
+from io import StringIO
+from typing import Any, Optional, Type, Union
+
+from .types import EventDict, Protocol, WrappedLogger
 
 
 try:
@@ -19,16 +22,18 @@ except ImportError:
 
 __all__ = ["ConsoleRenderer"]
 
+_IS_WINDOWS = sys.platform == "win32"
 
 _MISSING = "{who} requires the {package} package installed.  "
 _EVENT_WIDTH = 30  # pad the event name to so many characters
 
 
-def _pad(s, l):
+def _pad(s: str, length: int) -> str:
     """
-    Pads *s* to length *l*.
+    Pads *s* to length *lenght*.
     """
-    missing = l - len(s)
+    missing = length - len(s)
+
     return s + " " * (missing if missing > 0 else 0)
 
 
@@ -53,7 +58,27 @@ else:
     ) = DIM = RED = BLUE = CYAN = MAGENTA = YELLOW = GREEN = RED_BACK = ""
 
 
-class _ColorfulStyles(object):
+class _Styles(Protocol):
+    reset: str
+    bright: str
+    level_critical: str
+    level_exception: str
+    level_error: str
+    level_warn: str
+    level_info: str
+    level_debug: str
+    level_notset: str
+
+    timestamp: str
+    logger_name: str
+    kv_key: str
+    kv_value: str
+
+
+Styles = Union[_Styles, Type[_Styles]]
+
+
+class _ColorfulStyles:
     reset = RESET_ALL
     bright = BRIGHT
 
@@ -71,7 +96,7 @@ class _ColorfulStyles(object):
     kv_value = MAGENTA
 
 
-class _PlainStyles(object):
+class _PlainStyles:
     reset = ""
     bright = ""
 
@@ -89,30 +114,30 @@ class _PlainStyles(object):
     kv_value = ""
 
 
-class ConsoleRenderer(object):
+class ConsoleRenderer:
     """
-    Render `event_dict` nicely aligned, possibly in colors, and ordered.
+    Render ``event_dict`` nicely aligned, possibly in colors, and ordered.
 
-    If `event_dict` contains an ``exception`` key (for example from
+    If ``event_dict`` contains an ``exception`` key (for example from
     :func:`~structlog.processors.format_exc_info`), it will be rendered *after*
     the log line.
 
-    :param int pad_event: Pad the event to this many characters.
-    :param bool colors: Use colors for a nicer output.
-    :param bool force_colors: Force colors even for non-tty destinations.
+    :param pad_event: Pad the event to this many characters.
+    :param colors: Use colors for a nicer output.
+    :param force_colors: Force colors even for non-tty destinations.
         Use this option if your logs are stored in a file that is meant
         to be streamed to the console.
-    :param bool repr_native_str: When ``True``, :func:`repr()` is also applied
+    :param repr_native_str: When `True`, `repr` is also applied
         to native strings (i.e. unicode on Python 3 and bytes on Python 2).
-        Setting this to ``False`` is useful if you want to have human-readable
-        non-ASCII output on Python 2.  The `event` key is *never*
-        :func:`repr()` -ed.
-    :param dict level_styles: When present, use these styles for colors. This
+        Setting this to `False` is useful if you want to have human-readable
+        non-ASCII output on Python 2.  The ``event`` key is *never*
+        `repr` -ed.
+    :param level_styles: When present, use these styles for colors. This
         must be a dict from level names (strings) to colorama styles. The
         default can be obtained by calling
-        :meth:`ConsoleRenderer.get_default_level_styles`
+        `ConsoleRenderer.get_default_level_styles`
 
-    Requires the colorama_ package if *colors* is ``True``.
+    Requires the colorama_ package if *colors* is `True`.
 
     .. _colorama: https://pypi.org/project/colorama/
 
@@ -125,17 +150,20 @@ class ConsoleRenderer(object):
        ``colorama`` now initializes lazily to avoid unwanted initializations as
        ``ConsoleRenderer`` is used by default.
     .. versionchanged:: 19.2 Can be pickled now.
+    .. versionchanged:: 20.1 ``colorama`` does not initialize lazily on Windows
+       anymore because it breaks rendering.
     """
 
     def __init__(
         self,
-        pad_event=_EVENT_WIDTH,
-        colors=_has_colorama,
-        force_colors=False,
-        repr_native_str=False,
-        level_styles=None,
+        pad_event: int = _EVENT_WIDTH,
+        colors: bool = _has_colorama,
+        force_colors: bool = False,
+        repr_native_str: bool = False,
+        level_styles: Optional[Styles] = None,
     ):
         self._force_colors = self._init_colorama = False
+        styles: Styles
         if colors is True:
             if colorama is None:
                 raise SystemError(
@@ -145,9 +173,12 @@ class ConsoleRenderer(object):
                     )
                 )
 
-            self._init_colorama = True
-            if force_colors:
-                self._force_colors = True
+            if _IS_WINDOWS:  # pragma: no cover
+                _init_colorama(self._force_colors)
+            else:
+                self._init_colorama = True
+                if force_colors:
+                    self._force_colors = True
 
             styles = _ColorfulStyles
         else:
@@ -169,7 +200,7 @@ class ConsoleRenderer(object):
 
         self._repr_native_str = repr_native_str
 
-    def _repr(self, val):
+    def _repr(self, val: Any) -> str:
         """
         Determine representation of *val* depending on its type &
         self._repr_native_str.
@@ -182,15 +213,13 @@ class ConsoleRenderer(object):
         else:
             return repr(val)
 
-    def __call__(self, _, __, event_dict):
+    def __call__(
+        self, logger: WrappedLogger, name: str, event_dict: EventDict
+    ) -> str:
+
         # Initialize lazily to prevent import side-effects.
         if self._init_colorama:
-            if self._force_colors:
-                colorama.deinit()
-                colorama.init(strip=False)
-            else:
-                colorama.init()
-
+            _init_colorama(self._force_colors)
             self._init_colorama = False
         sio = StringIO()
 
@@ -207,15 +236,15 @@ class ConsoleRenderer(object):
         if level is not None:
             sio.write(
                 "["
-                + self._level_to_color[level]
+                + self._level_to_color.get(level, "")
                 + _pad(level, self._longest_level)
                 + self._styles.reset
                 + "] "
             )
 
         # force event to str for compatibility with standard library
-        event = event_dict.pop("event")
-        if not PY2 or not isinstance(event, string_types):
+        event = event_dict.pop("event", None)
+        if not isinstance(event, str):
             event = str(event)
 
         if event_dict:
@@ -260,22 +289,22 @@ class ConsoleRenderer(object):
         return sio.getvalue()
 
     @staticmethod
-    def get_default_level_styles(colors=True):
+    def get_default_level_styles(colors: bool = True) -> Any:
         """
         Get the default styles for log levels
 
-        This is intended to be used with :class:`ConsoleRenderer`'s
-        ``level_styles`` parameter.  For example, if you are adding
-        custom levels in your home-grown
-        :func:`~structlog.stdlib.add_log_level` you could do::
+        This is intended to be used with `ConsoleRenderer`'s ``level_styles``
+        parameter.  For example, if you are adding custom levels in your
+        home-grown :func:`~structlog.stdlib.add_log_level` you could do::
 
             my_styles = ConsoleRenderer.get_default_level_styles()
             my_styles["EVERYTHING_IS_ON_FIRE"] = my_styles["critical"]
             renderer = ConsoleRenderer(level_styles=my_styles)
 
-        :param bool colors: Whether to use colorful styles. This must match the
-            `colors` parameter to :class:`ConsoleRenderer`. Default: True.
+        :param colors: Whether to use colorful styles. This must match the
+            *colors* parameter to `ConsoleRenderer`. Default: `True`.
         """
+        styles: Styles
         if colors:
             styles = _ColorfulStyles
         else:
@@ -292,10 +321,21 @@ class ConsoleRenderer(object):
         }
 
 
+def _init_colorama(force: bool) -> None:
+    if force:
+        colorama.deinit()
+        colorama.init(strip=False)
+    else:
+        colorama.init()
+
+
 _SENTINEL = object()
 
 
-def set_exc_info(_, method_name, event_dict):
+def set_exc_info(
+    logger: WrappedLogger, method_name: str, event_dict: EventDict
+) -> EventDict:
+
     """
     Set ``event_dict["exc_info"] = True`` if *method_name* is ``"exception"``.
 
