@@ -9,12 +9,11 @@ networking engine.
 See also :doc:`structlog's Twisted support <twisted>`.
 """
 
-from __future__ import absolute_import, division, print_function
-
 import json
 import sys
 
-from six import PY2, string_types
+from typing import Any, Callable, Dict, Optional, Sequence, TextIO, Tuple
+
 from twisted.python import log
 from twisted.python.failure import Failure
 from twisted.python.log import ILogObserver, textFromEventDict
@@ -24,6 +23,7 @@ from ._base import BoundLoggerBase
 from ._config import _BUILTIN_DEFAULT_PROCESSORS
 from ._utils import until_not_interrupted
 from .processors import JSONRenderer as GenericJSONRenderer
+from .types import EventDict, WrappedLogger
 
 
 class BoundLogger(BoundLoggerBase):
@@ -41,20 +41,20 @@ class BoundLogger(BoundLoggerBase):
 
     """
 
-    def msg(self, event=None, **kw):
+    def msg(self, event: Optional[str] = None, **kw: Any) -> Any:
         """
         Process event and call ``log.msg()`` with the result.
         """
         return self._proxy_to_logger("msg", event, **kw)
 
-    def err(self, event=None, **kw):
+    def err(self, event: Optional[str] = None, **kw: Any) -> Any:
         """
         Process event and call ``log.err()`` with the result.
         """
         return self._proxy_to_logger("err", event, **kw)
 
 
-class LoggerFactory(object):
+class LoggerFactory:
     """
     Build a Twisted logger when an *instance* is called.
 
@@ -63,7 +63,7 @@ class LoggerFactory(object):
     >>> configure(logger_factory=LoggerFactory())
     """
 
-    def __call__(self, *args):
+    def __call__(self, *args: Any) -> WrappedLogger:
         """
         Positional arguments are silently ignored.
 
@@ -78,7 +78,7 @@ class LoggerFactory(object):
 _FAIL_TYPES = (BaseException, Failure)
 
 
-def _extractStuffAndWhy(eventDict):
+def _extractStuffAndWhy(eventDict: EventDict) -> Tuple[Any, Any, EventDict]:
     """
     Removes all possible *_why*s and *_stuff*s, analyzes exc_info and returns
     a tuple of ``(_stuff, _why, eventDict)``.
@@ -88,29 +88,36 @@ def _extractStuffAndWhy(eventDict):
     _stuff = eventDict.pop("_stuff", None)
     _why = eventDict.pop("_why", None)
     event = eventDict.pop("event", None)
-    if isinstance(_stuff, _FAIL_TYPES) and isinstance(event, _FAIL_TYPES):
+
+    if isinstance(_stuff, _FAIL_TYPES) and isinstance(  # type: ignore
+        event, _FAIL_TYPES
+    ):
         raise ValueError("Both _stuff and event contain an Exception/Failure.")
+
     # `log.err('event', _why='alsoEvent')` is ambiguous.
-    if _why and isinstance(event, string_types):
+    if _why and isinstance(event, str):  # type: ignore
         raise ValueError("Both `_why` and `event` supplied.")
+
     # Two failures are ambiguous too.
     if not isinstance(_stuff, _FAIL_TYPES) and isinstance(event, _FAIL_TYPES):
         _why = _why or "error"
         _stuff = event
-    if isinstance(event, string_types):
+
+    if isinstance(event, str):
         _why = event
+
     if not _stuff and sys.exc_info() != (None, None, None):
         _stuff = Failure()
+
     # Either we used the error ourselves or the user supplied one for
     # formatting.  Avoid log.err() to dump another traceback into the log.
     if isinstance(_stuff, BaseException) and not isinstance(_stuff, Failure):
         _stuff = Failure(_stuff)
-    if PY2:
-        sys.exc_clear()
+
     return _stuff, _why, eventDict
 
 
-class ReprWrapper(object):
+class ReprWrapper:
     """
     Wrap a string and return it as the ``__repr__``.
 
@@ -125,10 +132,10 @@ class ReprWrapper(object):
     Note the extra quotes in the unwrapped example.
     """
 
-    def __init__(self, string):
+    def __init__(self, string: str) -> None:
         self.string = string
 
-    def __eq__(self, other):
+    def __eq__(self, other: Any) -> bool:
         """
         Check for equality, just for tests.
         """
@@ -136,7 +143,7 @@ class ReprWrapper(object):
             isinstance(other, self.__class__) and self.string == other.string
         )
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return self.string
 
 
@@ -162,7 +169,12 @@ class JSONRenderer(GenericJSONRenderer):
     `plainJSONStdOutLogger` for pure-JSON logs.
     """
 
-    def __call__(self, logger, name, eventDict):
+    def __call__(  # type: ignore
+        self,
+        logger: WrappedLogger,
+        name: str,
+        eventDict: EventDict,
+    ) -> Tuple[Sequence[Any], Dict[str, Any]]:
         _stuff, _why, eventDict = _extractStuffAndWhy(eventDict)
         if name == "err":
             eventDict["event"] = _why
@@ -174,7 +186,9 @@ class JSONRenderer(GenericJSONRenderer):
         return (
             (
                 ReprWrapper(
-                    GenericJSONRenderer.__call__(self, logger, name, eventDict)
+                    GenericJSONRenderer.__call__(  # type: ignore
+                        self, logger, name, eventDict
+                    )
                 ),
             ),
             {"_structlog": True},
@@ -182,7 +196,7 @@ class JSONRenderer(GenericJSONRenderer):
 
 
 @implementer(ILogObserver)
-class PlainFileLogObserver(object):
+class PlainFileLogObserver:
     """
     Write only the the plain message without timestamps or anything else.
 
@@ -190,22 +204,21 @@ class PlainFileLogObserver(object):
     runit.
 
     :param file: File to print to.
-    :type file: file object
 
     .. versionadded:: 0.2.0
     """
 
-    def __init__(self, file):
+    def __init__(self, file: TextIO) -> None:
         self._write = file.write
         self._flush = file.flush
 
-    def __call__(self, eventDict):
+    def __call__(self, eventDict: EventDict) -> None:
         until_not_interrupted(self._write, textFromEventDict(eventDict) + "\n")
         until_not_interrupted(self._flush)
 
 
 @implementer(ILogObserver)
-class JSONLogObserverWrapper(object):
+class JSONLogObserverWrapper:
     """
     Wrap a log *observer* and render non-`JSONRenderer` entries to JSON.
 
@@ -217,10 +230,10 @@ class JSONLogObserverWrapper(object):
     .. versionadded:: 0.2.0
     """
 
-    def __init__(self, observer):
+    def __init__(self, observer: Any) -> None:
         self._observer = observer
 
-    def __call__(self, eventDict):
+    def __call__(self, eventDict: EventDict) -> str:
         if "_structlog" not in eventDict:
             eventDict["message"] = (
                 json.dumps(
@@ -231,10 +244,11 @@ class JSONLogObserverWrapper(object):
                 ),
             )
             eventDict["_structlog"] = True
+
         return self._observer(eventDict)
 
 
-def plainJSONStdOutLogger():
+def plainJSONStdOutLogger() -> JSONLogObserverWrapper:
     """
     Return a logger that writes only the message to stdout.
 
@@ -259,7 +273,7 @@ def plainJSONStdOutLogger():
     return JSONLogObserverWrapper(PlainFileLogObserver(sys.stdout))
 
 
-class EventAdapter(object):
+class EventAdapter:
     """
     Adapt an ``event_dict`` to Twisted logging system.
 
@@ -267,22 +281,28 @@ class EventAdapter(object):
     <https://twistedmatrix.com/documents/current/
     api/twisted.python.log.html#err>`_ behave as expected.
 
-    :param callable dictRenderer: Renderer that is used for the actual
-        log message.  Please note that structlog comes with a dedicated
-        :class:`JSONRenderer`.
+    :param dictRenderer: Renderer that is used for the actual log message.
+        Please note that structlog comes with a dedicated `JSONRenderer`.
 
     **Must** be the last processor in the chain and requires a *dictRenderer*
     for the actual formatting as an constructor argument in order to be able to
     fully support the original behaviors of ``log.msg()`` and ``log.err()``.
     """
 
-    def __init__(self, dictRenderer=None):
+    def __init__(
+        self,
+        dictRenderer: Optional[
+            Callable[[WrappedLogger, str, EventDict], str]
+        ] = None,
+    ) -> None:
         """
         :param dictRenderer: A processor used to format the log message.
         """
         self._dictRenderer = dictRenderer or _BUILTIN_DEFAULT_PROCESSORS[-1]
 
-    def __call__(self, logger, name, eventDict):
+    def __call__(
+        self, logger: WrappedLogger, name: str, eventDict: EventDict
+    ) -> Any:
         if name == "err":
             # This aspires to handle the following cases correctly:
             #   - log.err(failure, _why='event', **kw)
@@ -290,6 +310,7 @@ class EventAdapter(object):
             #   - log.err(_stuff=failure, _why='event', **kw)
             _stuff, _why, eventDict = _extractStuffAndWhy(eventDict)
             eventDict["event"] = _why
+
             return (
                 (),
                 {
